//-----------------------------------------------------------------------------
// File: CGameApp.h
//
// Desc: Game Application class, this is the central hub for all app processing
//
// Copyright (c) 1997-2002 Adam Hoult & Gary Simmons. All rights reserved.
//-----------------------------------------------------------------------------

#ifndef _CGAMEAPP_H_
#define _CGAMEAPP_H_

//-----------------------------------------------------------------------------
// CGameApp Specific Includes
//-----------------------------------------------------------------------------
#include "3D_Main.h"
#include "CTimer.h"
#include "CObject.h"
#include "CPlayer.h"
#include "CTerrain.h"

//-----------------------------------------------------------------------------
// Forward Declarations
//-----------------------------------------------------------------------------
class CCamera;

//-----------------------------------------------------------------------------
// Main Class Declarations
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// Name : CGameApp (Class)
// Desc : Central game engine, initialises the game and handles core processes.
//-----------------------------------------------------------------------------
class CGameApp
{
public:
    //-------------------------------------------------------------------------
	// Constructors & Destructors for This Class.
	//-------------------------------------------------------------------------
	         CGameApp();
	virtual ~CGameApp();

	//-------------------------------------------------------------------------
	// Public Functions for This Class
	//-------------------------------------------------------------------------
 //   LRESULT     DisplayWndProc( HWND hWnd, UINT Message, WPARAM wParam, LPARAM lParam );
	bool        InitInstance(HWND hwnd, CRect rc);
    void        FrameAdvance();
	bool        ShutDown();
	void		SetSize(bool isMin, CRect rc);
	void		Mouse_Lift_Down();
	void		Mouse_Lift_Up();
private:
    //-------------------------------------------------------------------------
	// Private Functions for This Class
	//-------------------------------------------------------------------------
    bool        BuildObjects      ( );
	void        ReleaseObjects    ( );
	void        ChangeDevice      ( );
    void        SetupGameState    ( );
    void        SetupRenderStates ( );
    void        AnimateObjects    ( );
    void        ProcessInput      ( );
    bool        InitDirect3D      (CRect rc);
	
    D3DFORMAT   FindDepthStencilFormat( ULONG AdapterOrdinal, D3DDISPLAYMODE Mode, D3DDEVTYPE DevType );
    
    //-------------------------------------------------------------------------
	// Private Variables For This Class
	//-------------------------------------------------------------------------
	CTerrain                m_Terrain;          // Simple terrain object (stores data)
	CPlayer                 m_Player;           // Player class used to manipulate our player object
    CCamera               * m_pCamera;          // A cached copy of the camera attached to the player

	D3DXMATRIX              m_mtxIdentity;      // A basic identity matrix
    D3DXMATRIX              m_mtxView;          // View Matrix

	CTimer                  m_Timer;            // Game timer
    ULONG                   m_LastFrameRate;    // Used for making sure we update only when fps changes.
    
    HWND                    m_hWnd;             // Main window HWND
    HICON                   m_hIcon;            // Window Icon
    HMENU                   m_hMenu;            // Window Menu
    
    bool                    m_bLostDevice;      // Is the 3d device currently lost ?
    bool                    m_bActive;          // Is the application active ?

	D3DPRESENT_PARAMETERS   m_D3DPresentParams; // Direct3D Present Parameters
    LPDIRECT3D9             m_pD3D;             // Direct3D Object
    LPDIRECT3DDEVICE9       m_pD3DDevice;       // Direct3D Device Object
    D3DFILLMODE             m_FillMode;         // Which fill mode are we using ?

    ULONG                   m_nViewX;           // X Position of render viewport
    ULONG                   m_nViewY;           // Y Position of render viewport
    ULONG                   m_nViewWidth;       // Width of render viewport
    ULONG                   m_nViewHeight;      // Height of render viewport

    POINT                   m_OldCursorPos;     // Old cursor position for tracking

    CObject_DX              m_Object;           // The object referencing the player mesh
    CMesh                   m_PlayerMesh;       // The player mesh (cube ;)
	
};

#endif // _CGAMEAPP_H_